////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  Project:  Embedded Learning Library (ELL)
//  File:     ModelLoadArguments.h (common)
//  Authors:  Ofer Dekel
//
////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include <utilities/include/CommandLineParser.h>

#include <string>

namespace ell
{
namespace common
{
    /// <summary> A struct that holds command line parameters for loading models. </summary>
    struct ModelLoadArguments
    {
        /// <summary> The file to read a model from. </summary>
        std::string inputModelFile = "";
    };

    /// <summary> A version of ModelLoadArguments that adds its members to the command line parser. </summary>
    struct ParsedModelLoadArguments : public ModelLoadArguments
        , public utilities::ParsedArgSet
    {
        /// <summary> Adds the arguments to the command line parser. </summary>
        ///
        /// <param name="parser"> [in,out] The parser. </param>
        void AddArgs(utilities::CommandLineParser& parser) override;

        /// <summary> Check arguments. </summary>
        ///
        /// <param name="parser"> The parser. </param>
        ///
        /// <returns> An utilities::CommandLineParseResult. </returns>
        utilities::CommandLineParseResult PostProcess(const utilities::CommandLineParser& parser) override;
    };
} // namespace common
} // namespace ell
